package com.example.midemo;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.Network;
import android.net.NetworkCapabilities;
import android.net.NetworkRequest;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;

import java.io.IOException;
import java.lang.reflect.Field;
import java.net.InetAddress;
import java.net.Socket;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MiuiDualWifi";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (!supportDualWifi(getApplicationContext())) {
            return;
        }

    }

    @Override
    protected void onResume() {
        super.onResume();
        requestSlaveWifiNetwork();
    }

    private void requestSlaveWifiNetwork() {
        if (!isSlaveWifiConnected(getApplicationContext())) {
            //TODO Provide Dual Wifi Ui guidance and reCheck
            Intent intent = new Intent("android.settings.DUAL_WIFI.WIFI_SETTINGS");
            startActivity(intent);
            return;
        }

        int slaveTransportType = getSlaveTransportType();
        if (slaveTransportType < 0) {
            return;
        }
        NetworkRequest nr = getNetworkRequestForType(slaveTransportType);
        SlaveCallback slaveNetworkCallback = new SlaveCallback();


        ConnectivityManager cm = (ConnectivityManager) getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        cm.requestNetwork(nr, slaveNetworkCallback);
    }

    private boolean supportDualWifi(Context context) {
        String cloudvalue = Settings.System.getString(context.getContentResolver(), "cloud_slave_wifi_support");
        if ("off".equals(cloudvalue)) {
            return false;
        }
        boolean support;
        try {
            support = context.getResources().getBoolean(
                    context.getResources().getIdentifier("config_slave_wifi_support", "bool", "android.miui"));
        } catch (Exception exception) {
            return false;
        }
        return support;
    }

    private boolean isSlaveWifiConnected(Context context) {
        String slaveWifiSsid = getDualWifiSsid(context);
        if (slaveWifiSsid == null || slaveWifiSsid.isEmpty()) {
            return false;
        } else {
            return true;
        }
    }

    private String getDualWifiSsid(Context context) {
        return Settings.System.getString(context.getContentResolver(), "slave_wifi_ssid");
    }

    private NetworkRequest getNetworkRequestForType(int type) {
        return new NetworkRequest.Builder()
                .addCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
                .addTransportType(type).build();
    }

    private int getSlaveTransportType() {
        int slaveTransportType = -1;
        try {
            Class networkCapabilitiesClass = Class.forName("android.net.NetworkCapabilities");
            Field field = networkCapabilitiesClass.getField("TRANSPORT_SLAVE_WIFI");
            slaveTransportType = (int) field.get(null);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return slaveTransportType;
    }

    private class SlaveCallback extends ConnectivityManager.NetworkCallback {
        @Override
        public void onAvailable(Network network) {
            Log.d(TAG, "SlaveCallback Network available: " + network.toString());
            Socket skS;
            try {
                InetAddress[] inetAddress = network.getAllByName("www.mi.com");
                for (InetAddress ia : inetAddress) {
                    Log.d(TAG, "SlaveCallback Network available: " + ia);
                }
                skS = network.getSocketFactory().createSocket("www.mi.com", 80);
            } catch (IOException e) {
                e.printStackTrace();
                return;
            }
            Log.d(TAG, "SlaveCallback skS = " + skS.toString());
        }
    }
}
